==> commerce_2c2p-1.0.0.tar.gz: commerce_2c2p/src/Plugin/Commerce/PaymentGateway/OffsiteRedirect2C2P.php <==
<?php

namespace Drupal\commerce_2c2p\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_price\MinorUnitsConverterInterface;
use Drupal\Component\Datetime\TimeInterface;
use GuzzleHttp\ClientInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Symfony\Component\HttpFoundation\Response;
use Firebase\JWT\JWT;
use Drupal\Core\Logger\LoggerChannelTrait;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Drupal\commerce_2c2p\Event\Commerce2C2PPaymentEvents;
use Drupal\commerce_2c2p\Event\PaymentSuccessEvent;

/**
 * Provides the Off-site Redirect payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "2c2p_offsite_redirect",
 *   label = "2C2P (Off-site Iframe)",
 *   display_label = "2C2P (Off-site Iframe)",
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_2c2p\PluginForm\OffsiteRedirect\PaymentOffsiteForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "amex", "dinersclub", "discover", "jcb", "maestro", "mastercard", "visa",
 *   },
 *   requires_billing_information = FALSE,
 * )
 */
class OffsiteRedirect2C2P extends OffsitePaymentGatewayBase {

  use LoggerChannelTrait;

  /**
   * Guzzle\Client instance.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * Constructs a new OffsiteRedirect2C2P object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\commerce_payment\PaymentTypeManager $payment_type_manager
   *   The payment type manager.
   * @param \Drupal\commerce_payment\PaymentMethodTypeManager $payment_method_type_manager
   *   The payment method type manager.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The http client.
   * @param \Drupal\commerce_price\MinorUnitsConverterInterface $minor_units_converter
   *   The minor units converter.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, PaymentTypeManager $payment_type_manager, PaymentMethodTypeManager $payment_method_type_manager, TimeInterface $time, ClientInterface $http_client, MinorUnitsConverterInterface $minor_units_converter = NULL, EventDispatcherInterface $event_dispatcher) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time, $minor_units_converter);

    $this->httpClient = $http_client;
    $this->eventDispatcher = $event_dispatcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('datetime.time'),
      $container->get('http_client'),
      $container->get('commerce_price.minor_units_converter'),
      $container->get('event_dispatcher')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'merchant_id' => '',
      'secret_key' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['merchant_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Merchant ID'),
      '#default_value' => $this->configuration['merchant_id'],
      '#required' => TRUE,
    ];

    $form['secret_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Secret key'),
      '#default_value' => $this->configuration['secret_key'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['merchant_id'] = $values['merchant_id'];
      $this->configuration['secret_key'] = $values['secret_key'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getApiUrl() {
    if ($this->getMode() == 'test') {
      return 'https://sandbox-pgw.2c2p.com/payment/4.1/';
    }
    else {
      return 'https://pgw.2c2p.com/payment/4.1/';
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {

    /* Check if order was completed in on notify, if so don't do anything. */
    if ($order->getState()->getId() !== 'completed') {
      $params = [
        "paymentToken" => $order->getData('commerce_2c2p_payment_token'),
        "merchantID" => $this->configuration['merchant_id'],
        "invoiceNo" => $order->getData('commerce_2c2p_invoice_no'),
      ];

      $jwt = JWT::encode($params, $this->configuration['secret_key']);
      $pt_response = $this->httpClient->post($this->getApiUrl() . 'paymentInquiry', [
        'json' => [
          "payload" => $jwt,
        ],
      ]);

      $pt_res_data = json_decode($pt_response->getBody()->getContents(), TRUE);
      $pt_res_payload = JWT::decode($pt_res_data['payload'], $this->configuration['secret_key'], [
        'HS256',
      ]);
      $this->getLogger('commerce_2c2p')->debug('2C2P paymentInquiry Response: ' . serialize($pt_res_payload));
      if ($pt_res_payload->respCode === "0000") {
        // Trigger payment success event.
        $event = new PaymentSuccessEvent($pt_res_payload, $order);
        $this->eventDispatcher->dispatch($event, Commerce2C2PPaymentEvents::PAYMENT_SUCCESS);
        $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
        $payment = $payment_storage->create([
          'state' => 'completed',
          'amount' => $order->getBalance(),
          'payment_gateway' => $this->parentEntity->id(),
          'order_id' => $order->id(),
          'remote_id' => $pt_res_payload->tranRef,
          'remote_state' => $pt_res_payload->respDesc,
        ]);
        $payment->save();
      }
      else {
        throw new PaymentGatewayException('Payment failed.');
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onNotify(Request $request) {
    $content = json_decode($request->getContent(), TRUE);
    $this->getLogger('commerce_2c2p')->debug('2C2P server-to-server Response: ' . serialize($content));
    if (!empty($content['payload'])) {
      try {
        $pt_res_payload = JWT::decode($content['payload'], $this->configuration['secret_key'], [
          'HS256',
        ]);
        $this->getLogger('commerce_2c2p')->debug('2C2P Payload Response notify ' . serialize($pt_res_payload));
        /** @var \Drupal\commerce_order\entity\OrderInterface $order */
        list(, $order_id) = explode('-', $pt_res_payload->invoiceNo);
        $order = $this->entityTypeManager->getStorage('commerce_order')->loadUnchanged($order_id);
        if (!$order) {
          $this->getLogger('commerce_2c2p')->error('Invalid order ID from gateway: @reference', ['@reference' => $pt_res_payload->invoiceNo]);
          throw new PaymentGatewayException('Invalid order ID from gateway.');
        }
        if ($pt_res_payload->respCode === "0000") {
          // Trigger payment success event.
          $event = new PaymentSuccessEvent($pt_res_payload, $order);
          $this->eventDispatcher->dispatch($event, Commerce2C2PPaymentEvents::PAYMENT_SUCCESS);
          $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
          $payment = $payment_storage->create([
            'state' => 'completed',
            'amount' => $order->getBalance(),
            'payment_gateway' => $this->parentEntity->id(),
            'order_id' => $order->id(),
            'remote_id' => $pt_res_payload->tranRef,
            'remote_state' => $pt_res_payload->respDesc,
          ]);
          $payment->save();
        }
        else {
          throw new PaymentGatewayException('Payment failed.');
        }
      }
      catch (\UnexpectedValueException $e) {
        return new Response('', 200);
      }
    }
  }

}

==> commerce_2c2p-1.0.1.tar.gz: commerce_2c2p/src/Plugin/Commerce/PaymentGateway/OffsiteRedirect2C2P.php <==
<?php

namespace Drupal\commerce_2c2p\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_price\MinorUnitsConverterInterface;
use Drupal\Component\Datetime\TimeInterface;
use GuzzleHttp\ClientInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Symfony\Component\HttpFoundation\Response;
use Firebase\JWT\JWT;
use Drupal\Core\Logger\LoggerChannelTrait;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Drupal\commerce_2c2p\Event\Commerce2C2PPaymentEvents;
use Drupal\commerce_2c2p\Event\PaymentSuccessEvent;

/**
 * Provides the Off-site Redirect payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "2c2p_offsite_redirect",
 *   label = "2C2P (Off-site Iframe)",
 *   display_label = "2C2P (Off-site Iframe)",
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_2c2p\PluginForm\OffsiteRedirect\PaymentOffsiteForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "amex", "dinersclub", "discover", "jcb", "maestro", "mastercard", "visa",
 *   },
 *   requires_billing_information = FALSE,
 * )
 */
class OffsiteRedirect2C2P extends OffsitePaymentGatewayBase {

  use LoggerChannelTrait;

  /**
   * Guzzle\Client instance.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * Constructs a new OffsiteRedirect2C2P object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\commerce_payment\PaymentTypeManager $payment_type_manager
   *   The payment type manager.
   * @param \Drupal\commerce_payment\PaymentMethodTypeManager $payment_method_type_manager
   *   The payment method type manager.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The http client.
   * @param \Drupal\commerce_price\MinorUnitsConverterInterface $minor_units_converter
   *   The minor units converter.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, PaymentTypeManager $payment_type_manager, PaymentMethodTypeManager $payment_method_type_manager, TimeInterface $time, ClientInterface $http_client, MinorUnitsConverterInterface $minor_units_converter = NULL, EventDispatcherInterface $event_dispatcher) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time, $minor_units_converter);

    $this->httpClient = $http_client;
    $this->eventDispatcher = $event_dispatcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('datetime.time'),
      $container->get('http_client'),
      $container->get('commerce_price.minor_units_converter'),
      $container->get('event_dispatcher')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'merchant_id' => '',
      'secret_key' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['merchant_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Merchant ID'),
      '#default_value' => $this->configuration['merchant_id'],
      '#required' => TRUE,
    ];

    $form['secret_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Secret key'),
      '#default_value' => $this->configuration['secret_key'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['merchant_id'] = $values['merchant_id'];
      $this->configuration['secret_key'] = $values['secret_key'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getApiUrl() {
    if ($this->getMode() == 'test') {
      return 'https://sandbox-pgw.2c2p.com/payment/4.1/';
    }
    else {
      return 'https://pgw.2c2p.com/payment/4.1/';
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {

    /* Check if order was completed in on notify, if so don't do anything. */
    if ($order->getState()->getId() !== 'completed') {
      $params = [
        "paymentToken" => $order->getData('commerce_2c2p_payment_token'),
        "merchantID" => $this->configuration['merchant_id'],
        "invoiceNo" => $order->getData('commerce_2c2p_invoice_no'),
      ];

      $jwt = JWT::encode($params, $this->configuration['secret_key']);
      $pt_response = $this->httpClient->post($this->getApiUrl() . 'paymentInquiry', [
        'json' => [
          "payload" => $jwt,
        ],
      ]);

      $pt_res_data = json_decode($pt_response->getBody()->getContents(), TRUE);
      $pt_res_payload = JWT::decode($pt_res_data['payload'], $this->configuration['secret_key'], [
        'HS256',
      ]);
      $this->getLogger('commerce_2c2p')->debug('2C2P paymentInquiry Response: ' . serialize($pt_res_payload));
      if ($pt_res_payload->respCode === "0000") {
        // Trigger payment success event.
        $event = new PaymentSuccessEvent($pt_res_payload, $order);
        $this->eventDispatcher->dispatch($event, Commerce2C2PPaymentEvents::PAYMENT_SUCCESS);
        $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
        $payment = $payment_storage->create([
          'state' => 'completed',
          'amount' => $order->getBalance(),
          'payment_gateway' => $this->parentEntity->id(),
          'order_id' => $order->id(),
          'remote_id' => $pt_res_payload->tranRef,
          'remote_state' => $pt_res_payload->respDesc,
        ]);
        $payment->save();
      }
      else {
        throw new PaymentGatewayException('Payment failed.');
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onNotify(Request $request) {
    $content = json_decode($request->getContent(), TRUE);
    $this->getLogger('commerce_2c2p')->debug('2C2P server-to-server Response: ' . serialize($content));
    if (!empty($content['payload'])) {
      try {
        $pt_res_payload = JWT::decode($content['payload'], $this->configuration['secret_key'], [
          'HS256',
        ]);
        $this->getLogger('commerce_2c2p')->debug('2C2P Payload Response notify ' . serialize($pt_res_payload));
        /** @var \Drupal\commerce_order\entity\OrderInterface $order */
        list(, $order_id) = explode('-', $pt_res_payload->invoiceNo);
        $order = $this->entityTypeManager->getStorage('commerce_order')->loadUnchanged($order_id);
        if (!$order) {
          $this->getLogger('commerce_2c2p')->error('Invalid order ID from gateway: @reference', ['@reference' => $pt_res_payload->invoiceNo]);
          throw new PaymentGatewayException('Invalid order ID from gateway.');
        }
        if ($pt_res_payload->respCode === "0000" && $order->getState()->getId() !== 'completed') {
          // Trigger payment success event.
          $event = new PaymentSuccessEvent($pt_res_payload, $order);
          $this->eventDispatcher->dispatch($event, Commerce2C2PPaymentEvents::PAYMENT_SUCCESS);
          $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
          $payment = $payment_storage->create([
            'state' => 'completed',
            'amount' => $order->getBalance(),
            'payment_gateway' => $this->parentEntity->id(),
            'order_id' => $order->id(),
            'remote_id' => $pt_res_payload->tranRef,
            'remote_state' => $pt_res_payload->respDesc,
          ]);
          $payment->save();
        }
        else {
          throw new PaymentGatewayException('Payment failed.');
        }
      }
      catch (\UnexpectedValueException|PaymentGatewayException) {
        return new Response('', 200);
      }
    }
  }

}
